@@ -11,17 +11,21 @@ from __future__ import (absolute_import, print_function)
 __all__ = ["PostgreSQLQueryProcessor"]
 
 import antlr4
+import sys
 
 from .PostgreSQLLexer import PostgreSQLLexer
 from .PostgreSQLParser import PostgreSQLParser
+from .PostgreSQLParserListener import PostgreSQLParserListener
 
 from ..exceptions import QueryError, QuerySyntaxError
 
-from .postgresqllisteners import ColumnKeywordFunctionListener,\
-        QueryListener, RemoveSubqueriesListener, SyntaxErrorListener,\
-        SchemaNameListener, PgSphereListener
+from .postgresqllisteners import PgSphereListener
 
-from .postgresqllisteners import parse_alias
+from ..common import parse_alias, process_column_name,\
+        get_column_name_listener, get_schema_name_listener,\
+        get_remove_subqueries_listener, get_query_listener,\
+        SyntaxErrorListener,\
+        get_column_keyword_function_listener
 
 
 class PostgreSQLQueryProcessor(object):
@@ -34,17 +38,16 @@ class PostgreSQLQueryProcessor(object):
         PostgreSQL query string.
 
     """
-    def __init__(self, query=None, indexed_objects=None):
+    def __init__(self, query=None):
         self.walker = antlr4.ParseTreeWalker()
-
         self.columns = set()
         self.keywords = set()
         self.functions = set()
         self.display_columns = []
         self.syntax_error_listener = SyntaxErrorListener()
-        self._indexed_objects = indexed_objects
+
         if query is not None:
-            self._query = self._strip_query(query)
+            self.set_query(query)
             self.process_query()
 
     def _extract_instances(self, column_keyword_function_listener):
@@ -69,7 +72,7 @@ class PostgreSQLQueryProcessor(object):
                         other_columns.append([j])
                 else:
                     select_list_columns.append(i[2])
-                alias = parse_alias(i[1].alias())
+                alias = parse_alias(i[1].alias(), '"')
                 if alias is not None:
                     column_aliases.append(alias)
                 ctx_stack.append(i)
@@ -177,7 +180,6 @@ class PostgreSQLQueryProcessor(object):
         # higer up in the tree structure. We'll revisit them later.
         missing_columns = []
         remove_column_idxs = []
-        extra_columns = []
 
         for i, col in enumerate(columns):
             c = col[0]
@@ -187,12 +189,12 @@ class PostgreSQLQueryProcessor(object):
             calias = c[1]
 
             # if * is selected we don't care too much
-            if c[0][0] is None and c[0][1] is None and c[0][2] == '*':
-                for slt in select_list_tables:
-                    extra_columns.append([[slt[0][0][0], slt[0][0][1], cname,
-                                           c[0][3]], calias])
-                remove_column_idxs.append(i)
-                continue
+            #  if c[0][0] is None and c[0][1] is None and c[0][2] == '*':
+                #  for slt in select_list_tables:
+                    #  extra_columns.append([[slt[0][0][0], slt[0][0][1], cname,
+                                           #  c[0][3]], calias])
+                #  remove_column_idxs.append(i)
+                #  continue
 
             # this can happen for example in ... WHERE EXISTS ... clauses
             if cname is None and calias is None:
@@ -322,7 +324,7 @@ class PostgreSQLQueryProcessor(object):
 
         return missing_columns
 
-    def process_query(self, replace_schema_name={}):
+    def process_query(self, replace_schema_name=None, indexed_objects=None):
         """
         Parses and processes the query. After a successful run it fills up
         columns, keywords, functions and syntax_errors lists.
@@ -331,8 +333,10 @@ class PostgreSQLQueryProcessor(object):
         # Antlr objects
         inpt = antlr4.InputStream(self.query)
         lexer = PostgreSQLLexer(inpt)
+        #  lexer.removeErrorListeners()
         stream = antlr4.CommonTokenStream(lexer)
         parser = PostgreSQLParser(stream)
+        lexer._listeners = [self.syntax_error_listener]
         parser._listeners = [self.syntax_error_listener]
 
         # Parse the query
@@ -340,12 +344,14 @@ class PostgreSQLQueryProcessor(object):
         if len(self.syntax_error_listener.syntax_errors):
             raise QuerySyntaxError(self.syntax_error_listener.syntax_errors)
 
-        if len(replace_schema_name.items()):
-            schema_name_listener = SchemaNameListener(replace_schema_name)
+        if replace_schema_name is not None:
+            schema_name_listener = get_schema_name_listener(
+                    PostgreSQLParserListener, '"')(replace_schema_name)
             self.walker.walk(schema_name_listener, tree)
             self._query = stream.getText()
 
-        query_listener = QueryListener()
+        query_listener = get_query_listener(PostgreSQLParserListener,
+                PostgreSQLParser, '"')()
         subquery_aliases = [None]
         keywords = []
         functions = []
@@ -371,8 +377,12 @@ class PostgreSQLQueryProcessor(object):
 
         # Iterate through subqueries starting with the lowerst level
         for ccc, ctx in enumerate(query_listener.select_expressions[::-1]):
-            remove_subquieries_listener = RemoveSubqueriesListener(ctx.depth())
-            column_keyword_function_listener = ColumnKeywordFunctionListener()
+            remove_subquieries_listener = get_remove_subqueries_listener(
+                    PostgreSQLParserListener, PostgreSQLParser)(ctx.depth())
+            #column_keyword_function_listener = ColumnKeywordFunctionListener()
+            column_keyword_function_listener = \
+                    get_column_keyword_function_listener(
+                            PostgreSQLParserListener, '"')()
 
             # Remove nested subqueries from select_expressions
             self.walker.walk(remove_subquieries_listener, ctx)
@@ -399,7 +409,8 @@ class PostgreSQLQueryProcessor(object):
                 join_using, column_aliases =\
                 self._extract_instances(column_keyword_function_listener)
 
-            tables.extend([i[0][0] for i in select_list_tables])
+            #tables.extend([i[0][0] for i in select_list_tables])
+            tables.extend([i[0] for i in select_list_tables])
 
             # Then we need to connect the column names s with tables and
             # databases
@@ -469,6 +480,13 @@ class PostgreSQLQueryProcessor(object):
                                          for i in mc])
                 raise QueryError("Unreferenced column(s): '%s'." % unref_cols)
 
+        # If we have indexed_objects, we are also accessing those. We
+        # need to add them into the columns stack:
+        if indexed_objects is not None:
+            for k, v in indexed_objects.items():
+                for vals in v:
+                    touched_columns.append([[vals[0][0], vals[0][1], vals[2], None], None])
+
         touched_columns = set([tuple(i[0]) for i in touched_columns])
 
         # extract display_columns
@@ -503,15 +521,25 @@ class PostgreSQLQueryProcessor(object):
         self.display_columns = [(i[0].lstrip('"').rstrip('"'),
                                 list(self._strip_column(i[1])))
                                 for i in display_columns]
-        self.tables = [[i[0].lstrip('"').rstrip('"'),
-                        i[1].lstrip('"').rstrip('"')] for i in tables]
+
+        self.tables = list(set([tuple([i[0][0].lstrip('"').rstrip('"')
+                        if i[0][0] is not None else i[0][0],
+                        i[0][1].lstrip('"').rstrip('"')
+                        if i[0][1] is not None else i[0][1]]) for i in tables]))
 
         # If there are any pg_sphere objects that are indexed we need
         # to replace the ADQL translated query parts with the indexed column
         # names
-        if self._indexed_objects is not None:
-            pg_sphere_listener = PgSphereListener(columns,
-                                                  self._indexed_objects)
+        if indexed_objects is not None:
+            # we need to correctly alias 'pos' columns
+            for k, v in indexed_objects.items():
+                indexed_objects[k] = list([list(i) for i in v])
+                for i, vals in enumerate(v):
+                    for t in tables:
+                        if vals[0][0] == t[0][0] and vals[0][1] == t[0][1] and t[1] is not None:
+                            indexed_objects[k][i][2] = t[1] + '.' + indexed_objects[k][i][2]
+
+            pg_sphere_listener = PgSphereListener(columns, indexed_objects)
             self.walker.walk(pg_sphere_listener, tree)
             for k, v in pg_sphere_listener.replace_dict.items():
                 self._query = self._query.replace(k, v)
@@ -525,6 +553,12 @@ class PostgreSQLQueryProcessor(object):
         return self._query
 
     def _strip_query(self, query):
+        if sys.version_info[0] < 3:
+            try:
+                query = unicode(query, 'utf-8')
+            except TypeError:
+                # already unicode so we don't do anything
+                pass
         return query.lstrip('\n').rstrip().rstrip(';') + ';'
 
     def _strip_column(self, col):
@@ -537,7 +571,6 @@ class PostgreSQLQueryProcessor(object):
     def set_query(self, query):
         """
         Helper to set the query string.
-
         """
         self.columns = set()
         self.keywords = set()
